#!/u#!/usr/bin/env python3
import builtins
from impQ import *
import time
import serial
GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)
COL = getPinOut('COL')          # 3,4,5pin
ROW = getPinOut('ROW')
ssR = getPinOut('SS')           # 6pin
spi = getPinOut('SPI')          # 6,7,8,9pin
selSen = getPinOut('SEL_SEN')   # 10pin
oeIled = getPinOut('OE_ILED')     # 11pin
fix = getPinOut('FIX_CLED')     # 12pin
oeCled = getPinOut('OE_CLED')   # 13pin
pwmCled = getPinOut('PWM')      # 14pin
COL_LEN = 8
ROW_LEN = builtins.rowLen

def Elapse(elaSec):
    start = time.time()
    def closure():
        return time.time() - start > elaSec
    def rstEla():
        nonlocal start
        start = time.time()
    return closure, rstEla

ser = serial.Serial(
    port='/dev/ttyS0', #Replace ttyS0 with ttyAM0 for Pi1,Pi2,Pi0
    baudrate = 57600,
    parity=serial.PARITY_NONE,
    stopbits=serial.STOPBITS_ONE,
    bytesize=serial.EIGHTBITS,
    timeout=1
)

# ********************************* Sensor *************************************
class Sensor():
    cri = [45,120,220,400,500,700]
    # cri = [90,275,535,990,1750,2800]

    VON_CORRECTION = 20                # for GND Level correction

    def __init__(self):
        self.__adVal = 0
        self.__value = 0

    @property
    def AD(self):
        return self.__adVal

    @property
    def Value6(self):                  # 1 ~ 6
        return self.__value

    @Value6.setter
    def Value6(self, adVal):
        for i in range(len(self.cri)):
            if adVal < self.cri[i] + self.VON_CORRECTION:
                self.__value = i
                return
        self.__value = len(self.cri)

    @classmethod
    def init(cls): # *********** init *************
        cls.adAryInit = [[0 for _ in range(COL_LEN)] for _ in range(ROW_LEN)]
        print('cri:', cls.cri)
        cls.sens = [[Sensor() for _ in range(COL_LEN)] for _ in range(ROW_LEN)]
#           *********** calibration ***********
        shift = 8
        for i in range(1 << shift):     # add 256 times
            cls.setAd(True)
        for r in range(ROW_LEN):        # >> 8
            for c in range(COL_LEN):
                cls.adAryInit[r][c] = cls.adAryInit[r][c] >> shift

    @classmethod
    def setAd(cls, isInit=False):       #
        ''' row -> col '''
        for r in range(ROW_LEN):
            outNumBit(ROW, r)           # set row
            for c in range(COL_LEN):
                outNumBit(COL, c)       # set col
                cls.__spiAdc(r, c, isInit)

    @classmethod
    def sumTpl(cls, tpl2):              # tpl2:((r, c),(0,0),(0,1))
        if type(tpl2) == int:              # sumTpl((7))
            return sum([s.Value6 for s in Sensor.sens[tpl2]])
        elif type(tpl2[0]) == int:
            return Sensor.sens[tpl2[0]][tpl2[1]].Value6
        return sum([Sensor.sens[v0][v1].Value6 for (v0, v1) in tpl2])

    @classmethod                        # finger ditection ((0,2)), True, 5 )
    def finDet(cls, rc_lar_cri):        # *** ((position), on_True, criteria) ***
        #print(rc_lar_cri)
        for tpl2, isLarge, cri in rc_lar_cri:
            if (cls.sumTpl(tpl2) > cri)  ^ isLarge: return False
        return True

    @classmethod
    def senLst(cls, tpl2):
        return [cls.sens[r][c] for (r, c) in tpl2]

    @classmethod
    def getAdLst(cls, r=0, c=8, shift=0):
        lst = [s.AD>>shift for s in cls.sens[r]]
        return lst[0:c]

    @classmethod
    def getVal(cls, r, c):
        return cls.sens[r][c].Value6

    @classmethod
    def getAD(cls, r, c=-1, isInv=False):
        if c == -1:                     # r:(r, c)
            c = r[1]
            r = r[0]
            isInv = True
        if isInv:
            return 4000 - cls.sens[r][c].__adVal
        else:
            return cls.sens[r][c].__adVal

    @classmethod
    def __spiAdc(cls, r, c, isInit):
        ''' get AD-value and set it to sens[r][c] '''
        ledSta = getPinIn('OE_CLED')    # buffering oe
        oeCled(False)                   # leds.off
        cls.__adcOn(True, False)        # oeIled:off
        Voff = cls.__getAdVal(c)
        cls.__adcOn(True)               # oeIled:on
        Von = cls.__getAdVal(c)
        cls.__adcOn(False)
        oeCled(ledSta)                  # undo oe
        if isInit:
            cls.adAryInit[r][c] += noMinus(Von - Voff - (Voff>>2))
        else:
            adVal = noMinus(Von - Voff - (Voff>>2) - cls.adAryInit[r][c])
            cls.sens[r][c].__adVal = adVal
            cls.sens[r][c].Value6 = adVal

    @classmethod
    def __getAdVal(cls, c):
        adc = spi.xfer2([ 6 | (c >> 2), c << 6, 0])
        return ((adc[1] & 0x0f) << 8) + adc[2]

    @classmethod
    def __adcOn(cls, isSelSen, isIled = None):
        if isIled is None:isIled = isSelSen
        selSen(isSelSen)                # selsen(10) True:sensor on
        oeIled(isIled)

# ********************************** CLed **************************************
class CLed():
    ''' Serial Parallel converter for color LED '''
    indiColor = 6
    __indiBlk = False                   # indicator blink
    indiOn = True                       # indicator enabled

    def __init__(self):
        pass

    def ovWrt(self, tpl2, indiOff=False):
        ssR(False)                      # ss on
        oeCled(False)                   # oe off
        sLst = [0 for _ in range(COL_LEN)]
        for r in range(ROW_LEN):
            outNumBit(ROW, r)           # set row
            for c in range(COL_LEN):
                sLst[c] = (Sensor.getVal(r,c) if tpl2[r][c] == 0 else tpl2[r][c])
            self.setSb(sLst, r, True)
        ssR(True)                       # ss off
        oeCled(True)                    # oe on

    def turnOn(self, tpl2=-1, indiOff=False):# val:int/tuple
# ***** value preperation *****
        ssR(False)                      # ss on
        oeCled(False)                   # oe off
        for r in range(ROW_LEN):
            outNumBit(ROW, r)           # set row
            if type(tpl2) == int:
                if tpl2 == -1:          # normal
                    sLst = [s.Value6 for s in Sensor.sens[r]]
                else:                   # paint all surface
                    sLst = [tpl2 for _ in range(COL_LEN)]
            else:                       # tpl2 is tuple ex:tpl = ((1,2,3,4,5,6,7,6),(1,2,3,4,5,6,7,6))
                sLst = [tpl2[r][c] if tpl2[r][c] > 0 else 0 for c in range(COL_LEN)]
            self.indiOn &= (sum(sLst) == 0)
            self.setSb(sLst, r)
        self.indiOn = True
        ssR(True)                       # ss off
        oeCled(True)                    # oe on

    def setSb(self, lst, row, forceIndi=False):       # set single board, cube-indicator, row:indicator row detection
#         colorTbl = (0,4,5,1,3,2,6) #IHS10
        colorTbl = (0,1,3,2,6,4,5) #IHS11
        val24 = 0
        for c in range(COL_LEN):     # col7+col6+col5+..col0
            isIndi =((lst[c] == 0) and self.indiOn and self.__indiBlk) or forceIndi
            if (row == ROW_LEN - 1) and (c == COL_LEN - 1) and isIndi:
                cVal = self.indiColor
            else:
                cVal = lst[c]         # sensor value
            val24 += colorTbl[cVal] << c * 3
        wLst = [(val24 >> b * COL_LEN) & 0xff for b in range(2, -1, -1)]
        spi.xfer2(wLst)                 # [col7col6co] [l5col4col3co] [l2col1col0]
        fix(True)                       # latch clock and output color LED
        fix(False)

    @classmethod
    def setIndi(cls, isOn):
       cls.__indiBlk = isOn

    @classmethod
    def setIndiColor(cls, color = 6):
        cls.indiColor = color

cled = CLed()

if __name__ == '__main__':
    print('Sensor_CLED is running now!')