#!/u#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import builtins
builtins.rowLen = 1                     # *** set ROW_LEN ***
import serial
import threading
from Sensor_CLED import *
from swipe import *

def makeTimerGpiof(gpiof, tplSec):
    cnt = 0
    def closure():
        nonlocal cnt
        gpiof((cnt % 2)==0)
        threading.Timer(tplSec[cnt], closure).start()
        cnt = 0 if cnt == len(tplSec) - 1 else cnt + 1
    return closure

ledPwm = makeTimerGpiof(pwmCled, (10e-3, 10e-3))    #on2m / off18m
ledPwm()
indiBlk = makeTimerGpiof(CLed.setIndi, (0.4, 0.6))  #on / off
indiBlk()
# ******************************* OledMotion *********************************
class OledMotion:                       # 0, 1, 2
    CRI_AD = 16
    CRI_NOIZE = 10
    CRI_HOVER = 40
    CNT_HOVER = 50
    CNT_DIR = 4
    LAST_CRI = 2
    DIR = {0:'right', 1:'left', 2:'low', 3:'high'}

    def __init__(self): # 3, 4, 5
        self.hoverIdx = -1
        self.hoverCnt = 0
        self.stDir = ''
        self.fix = False                # fix flag
        self.idxF = -1
        self.dirF = [0 for _ in range(4)] # LRLowHi, direction first
        self.bl2d = [[False,False,False,False],[False,False,False,False],[False,False,False,False],[False,False,False,False]] # bool list 2D
        self.i_ = 0

    def init(self, isStr=True):
        for i in range(4):
            self.dirF[i] = 0

    def hover(self, adl):               # detect over center sensor (adl[1], adl[4])
        def isHover(maxVal=-1):
            if maxVal == -1:
                maxVal = max(adl)
                self.hoverIdx = adl.index(maxVal)
            lst = [i for i in adl if i < maxVal]
            if len(lst) == 0:
                return False
            max2 = max(lst)
            return maxVal > self.CRI_HOVER and max2 < self.CRI_NOIZE
        if self.hoverCnt > 0:
            if isHover(adl[self.hoverIdx]):
                #print(adl, self.hoverCnt)
                self.hoverCnt +=1
                if self.hoverCnt == self.CNT_HOVER:
                    self.hoverCnt = 0
                    return True
            else:
                #print('NG', adl)
                self.hoverCnt = 0
        elif isHover():                 # self.hoverCnt : 0
            self.hoverCnt = 1
        return False

    def notice(self, adl):              # motion detecrion and return direction string
        adlSum = sum(adl)               # set LRLH
        if adlSum < self.CRI_AD:
            rev = self.fix
            if self.fix:
                self.fix = False
                dir = [0 for _ in range(4)]
                for lst in self.bl2d:
                    self.cntTrueIdx(lst, dir)
                maxDir = max(dir)
                idx = self.idxLst2Fst(dir.index(maxDir)) # convert last position to first position
                if (idx != self.idxF) and (maxDir <= self.LAST_CRI):
                    idx = self.idxF
                self.stDir = self.DIR[idx]
            self.init()
            return rev
        if self.fix:
            self.scrutiny(adl, self.dirF)
        else:
            self.scrutiny(adl, self.dirF)
            maxV = max(self.dirF)
            if maxV >= self.CNT_DIR:
                self.idxF = self.dirF.index(maxV)
                # self.stDir = self.DIR[self.idxF]
                self.fix = True
        return False

    def scrutiny(self, adl, lst):
        getLine = lambda tpl : len(tuple(filter(lambda x: x > self.CRI_AD, tpl))) >= 2
        Left = (adl[0], adl[3])
        Right = (adl[2], adl[5])
        Low = tuple(adl[0:3])
        High = tuple(adl[3:])
        blTpl = (Left,Right,Low,High)   # bool tuple
        blLst = [False,False,False,False]
        for i in range(4):
            #blLst.append(getLine(blTpl[i]))# blLst:[True,False,False,False,] example
            blLst[i] = getLine(blTpl[i])
        if self.fix:                    # last
            for i in range(4):
                self.bl2d[self.i_][i] = False
            self.i_ += 1
            if self.i_ == 4:
                self.i_ = 0
        else:                           # first
            self.cntTrueIdx(blLst, lst)

    def cntTrueIdx(self, blLst, cntIdx):# bool list, count index list
        cnt = blLst.count(True)
        if cnt == 1:
            idx = blLst.index(True)
            cntIdx[idx] += 1

    def idxLst2Fst(self, idxL):         # reverse direction (convert last index to first index)
        if idxL in (0,1):
            return 0 if idxL == 1 else 1
        else:
            return 3 if idxL == 4 else 4

# ********************************* main *************************************
def setupWait():
    flg = False
    def closure(lst):
        nonlocal flg
        if (not any(lst)) and (not flg):
            print('start')
            flg = True
        return flg
    return closure

if __name__ == '__main__':
    # logging.basicConfig(level=logging.DEBUG)
    cled = CLed()
    Sensor.init()          # *** initialize Sensor ***
    print('******************* START ********************')
    try:
        device = get_device()
        swp = Swipe(device)
        swp.isOn = True
        swp.drift('')                                   # OLED text initialization
        swp.isOn = False
        cled.turnOn(5)
        time.sleep(1)
        om = OledMotion()
        setWait = setupWait()
        ela, elaRst = Elapse(2.5)                       # check elapsed time
        GPIO.setup(1, GPIO.IN, pull_up_down=GPIO.PUD_UP) # ** Switch for debug **
        while True:
            swpOn = swp.isOn
            swp.drift(om.stDir)                         # if swp.isOn:True OLED strings drift
            if swpOn and (swpOn != swp.isOn):           # swpOn : True -> False
                om.init()
            if not swp.isOn:
                Sensor.setAd()                          # 2, 1, 0
                cled.turnOn()                           # 5, 4, 3
                ad = Sensor.getAdLst(c=6, shift=2)
                adLst = (ad[2],ad[1],ad[0],ad[5],ad[4],ad[3])
                if not setWait(adLst):
                    continue
#             print(adLst[2],adLst[1],adLst[0],adLst[5],adLst[4],adLst[3])
                if om.hover(adLst):
                    row = 0 if om.hoverIdx < 3 else 1
                    swp.drawRect(row, True)               # draw rectangle outline in row:1
                    time.sleep(1.5)
                    swp.drawRect(row, False)              # erase rectangle outline in row:1
                    time.sleep(0.1)
                elif om.notice(adLst):                    # motion detection
                    swp.isOn = True
                if not GPIO.input(1):                     # if sw on
                    print(adLst)

    except KeyboardInterrupt:
        print("\nCtl+C")
    except Exception as e:
        print(str(e), "C")
    finally:
        cled.turnOn(0)
        spi.close()
