#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import time
from pathlib import Path
from datetime import datetime

if os.name != 'posix':
    sys.exit(f'{os.name} platform is not supported')

from demo_opts import get_device
from luma.core.render import canvas
from PIL import ImageFont

try:
    import psutil
except ImportError:
    print("The psutil library was not found. Run 'sudo -H pip install psutil' to install it.")
    sys.exit()

class AryXY:
    def __init__(self, strIdx, init, lattice):
        self.strIdx = strIdx            # str3X or str3Y : str3[str3Y][str3X][row]
        self.init = init
        self.delta = 0
        self.lattice = lattice          # x:128, y:64

class Swipe:
    ud = ('low', 'high')                # up down
    lr = ('left', 'right')              # left right
    # str3 = ((('T0abc', 'T0xyz'), ('Good morning', 'troublemakers'), (u'おはよう', u'ようこそ'), (u'早上好', u'欢迎'),('T2abc', 'T2xyz')),
    #         (('U0abc', 'U0xyz'), ('Hello', 'Really like'), (u'こんにちは', u'大好き'), (u'你好', u'很喜欢'),('U2abc', 'U2xyz')),
    #         (('V0abc', 'V0xyz'), ('good evening', 'Thank you'), (u'こんばんわ', u'ありがとう'), (u'晚上好', u'谢谢'),('V2abc', 'V2xyz')),
    #         (('W0abc', 'W0xyz'), ('good bye', 'Good luck'), (u'さようなら', u'幸運を'), (u'再见', u'祝你好运'),('W2abc', 'W2xyz')),
    #         (('X0abc', 'X0xyz'), ('how are you?', 'Fine'), (u'お元気ですか', u'元気です'), (u'你好吗？', u'美好的'),('X2abc', 'X2xyz')))
    # str3 = (((u'おはよう', 'Good morning'),   (u'クレージーな', "Here's to the"), (u'本当に', 'can change'), (u'世界を', 'the world,'), (u'ようこそ', 'Welcome')),
    #         ((u'こんにちは', 'Hello'),        (u'人たちがいる', 'crazy ones'), (u'人達こそが', 'think they'), (u'変えて', 'are the ones'), (u'大好き', 'Really like')),
    #         ((u'こんばんわ', 'Good evening'), (u'反逆者', 'The rebels'), (u'本気で信じる', ' enough to'), (u'いるのだから', 'who do'), (u'ありがとう', 'Thank you')),
    #         ((u'お元気ですか','How are you?'),(u'厄介者', 'troublemakers'), (u'変えられると', 'who are crazy'), (u'発想を', 'Think'), (u'幸運を', 'Good luck')),
    #         ((u'さようなら', 'Good bye'),     (u'天才だと思う', 'we see genius'), (u'自分が世界を', 'the people'), (u'変える', 'different'), (u'気をつけて', 'Take Care')))
    str3 = (((u'センサーの↑', "The sensor's"), (u'感染症の←', 'when we go'),  (u'経験を↑', 'through'),      (u'います。←', 'will be'),       (u'ありがとう←', 'Thank You')),
        ((u'赤外線は↑', 'infrared'),           (u'などの↓', 'in the future'), (u'経て↑', 'infections'),     (u'私は信じて↓', 'will be'),     (u'ございます←', 'very much!')),
        ((u'透過OLEDを↑', 'light can'),        (u'コロナ↓', 'I think'),       (u'清潔に↑', 'such as'),      (u'が役立つと↓', 'this sensor'), (u'メーカー←', 'Maker')),
        ((u'貫通して↑', 'detect'),             (u'できます。↓', 'the OLED.'), (u'なって↑', 'COVID-19 and'), (u'このセンサー↓', 'I believe'), (u'フェア←', 'Faire')),
        ((u'物体を検出←', 'objects'),          (u'することが↓', 'through'),   (u'いく←', 'become'),         (u'未来に↓', 'cleaner'),         (u'京都2023←', 'Kyoto 2023')))
#     str3 = (((u'センサーの', "The sensor's"), (u'感染症の', 'when we go'),  (u'経験を', 'through'),      (u'います。', 'useful'),       (u'本当に', 'can change')),
#         ((u'赤外線は', 'infrared'),           (u'などの', 'in the future'), (u'経て', 'infections'),     (u'私は信じて', 'will be'),     (u'役立つと', 'useful')),
#         ((u'清潔な', 'cleaner'),        (u'コロナ', 'I think'),       (u'清潔に', 'such as'),      (u'が役立つと', 'this sensor'), (u'未来に', 'future')),
#         ((u'貫通して', 'detect'),             (u'できます。', 'the OLED.'), (u'なって', 'COVID-19 and'), (u'このセンサー', 'I believe'), (u'変えられると', 'who are crazy')),
#         ((u'物体を検出', 'objects'),          (u'することが', 'through'),   (u'いく', 'become'),         (u'未来に', 'cleaner'),         (u'自分が世界を', 'the people')))

    @staticmethod
    def getIdx(i, le):
        if i == le:
            return 0
        return le - 1 if i < 0 else i

    @classmethod
    def getStr3(cls, y, x, row):
        yLen = len(cls.str3)
        xLen = len(cls.str3[0])
        ix = cls.getIdx(x, xLen)
        iy = cls.getIdx(y, yLen)
        return cls.str3[iy][ix][row]

    def __init__(self, device):         # device:128*64
        self.device = device
        self.fontSize = 18
        font_path = str(Path(__file__).resolve().parent.joinpath('fonts', 'code2000.ttf'))
        self.font2 = ImageFont.truetype(font_path, self.fontSize)
        self.spd = 8                    # speed
        self.isOn = False
        self.x = AryXY(0, 0, 128)       # 4 strIdx, init, lattice
        self.y = AryXY(0, 2, 64)
        self.rec = [0,0,0,0] # draw.rextangle([x0,y0,x1,y1])

    def drift(self, stDir, rectRow=-1):  # rectRow:rectangle outline 0 or 1(row)
        if not self.isOn:
            return
        if stDir == '':
            self.shift(0, 0, rectRow)            # stop!
        if stDir in self.lr:            # leftright : x
            di = 1 if stDir == 'right' else -1
            self.shift(0, di)           # di: direction(1 or -1)
        if stDir in self.ud:            # updown : y
            di = 1 if stDir == 'low' else -1
            self.shift(1, di)           # di: direction(1 or -1)

    def drawRect(self, row, ison):
        row1 = row if ison else -1
        self.shift(0, 0, row1)

    def shift(self, xyNum, di, rectRow=-1): # xyNum: 0:x, 1:y rectRow: 0 or 1(row)
        def dr(row, strIdx, suffix=''): # row:0 or 1,
            if di == 0:                 # stop!
                x = self.x.init
                y = self.y.init + (self.y.lattice >> 1) * row
                st = self.getStr3(self.y.strIdx, self.x.strIdx, row) + suffix
                # st = self.str3[self.y.strIdx][self.x.strIdx][row] + suffix
            elif xyNum == 0:            # x
                x = self.x.lattice * strIdx + self.x.init + self.x.delta
                y = self.y.init + (self.y.lattice >> 1) * row
                # st = self.str3[self.y.strIdx][strIdx + self.x.strIdx][row] + suffix
                st = self.getStr3(self.y.strIdx, strIdx + self.x.strIdx, row) + suffix
            elif xyNum == 1:            # y
                x = self.x.init
                y = self.y.lattice * strIdx + self.y.init + self.y.delta + (self.y.lattice >> 1) * row
                st = self.getStr3(strIdx + self.y.strIdx, self.x.strIdx, row) + suffix
                # st = self.str3[strIdx + self.y.strIdx][self.x.strIdx][row] + suffix
            draw.text((x, y), st, font=self.font2, fill="white")
        xy = self.x if xyNum == 0 else self.y # AryXY object
        with canvas(self.device) as draw:
            dr(0, -1)                   # , str(xy.delta)
            dr(1, -1)
            dr(0, 0)                    # , str(xy.delta)
            dr(1, 0)
            dr(0, 1)                    # , str(xy.delta)
            dr(1, 1)
            if rectRow >= 0:
                x, y = self.x.init, self.y.init + (self.y.lattice >> 2) * rectRow
                st = self.getStr3(self.y.strIdx, self.x.strIdx, rectRow)
                y2 = (self.y.lattice >> 2) * rectRow
                rx0, ry0 = x, y + y2 + 2 # (x0, y0)
                w, h = len(st) * 6, 10
                ratio = 3 if ord(st[0]) > 256 else 2
                rx1, ry1 = x + w * ratio, y + y2 + h * 1.8 + 2 # (x1, y1)
                print(rectRow, rx0, ry0, rx1, ry1)
                draw.rectangle((rx0, ry0, rx1, ry1), outline="white")
        xy.delta += di * self.spd
        if abs(xy.delta) > xy.lattice:
            xy.delta = 0
            xylen = len(self.str3[0]) if xyNum == 0 else len(self.str3[0])
#             xy.strIdx -= di
#             di = self.getIdx(di, xylen)
            xy.strIdx = self.getIdx(xy.strIdx - di, xylen)
            with canvas(self.device) as draw:
                dr(0, 0)
                dr(1, 0)
            self.isOn = False

def Counter(maxV):
    cnt = 0
    def closure():
        nonlocal cnt
        cnt += 1
        if cnt == maxV:
            cnt = 0
            return maxV
        else:
            return cnt
    return closure

def makeDir():
    dir1 = ('left', 'left', 'left', 'low', 'right', 'high')#('left', 'low', 'right', 'high', '') # ('left', 'right') ('low', 'high')
    idx = 0
    def closure():
        nonlocal idx
        i = idx
        idx += 1
        if idx == len(dir1):
            idx = 0
        return dir1[i]
    return closure

def main():
    swp = Swipe(device)
    di = makeDir()
    stDir = ''
    cnt = Counter(10)

    swp.isOn = True
    swp.shift(0, 0, 0)
    time.sleep(3)

    swp.isOn = False
    while True:
        if not swp.isOn:
            stDir = di()
            swp.isOn = True
            swp.drift(stDir)
            time.sleep(1)
            print(stDir)
        else:
            swp.drift(stDir)
            if stDir == '':
                if cnt() == 10:
                    swp.isOn = False
            print('*')


if __name__ == "__main__":
    try:
        device = get_device()
        main()
    except KeyboardInterrupt:
        pass